package net.core.editor.roles.diagram.custom.repl;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.GregorianCalendar;

import org.eclipse.emf.common.util.URI;

import net.core.editor.roles.diagram.custom.util.Bridge;

import static conexp.contrib.java.Main.*;

public class CoreRunner {
	//parameters
	public static final String PRE = "core> ";
	public static final String EXIT = "exit";
	public static final String HELP = "help";
	public static final String ALL = "all";
	public static final String PROPER = "p";
	//commands
	public static final String PRINT = "print";
	public static final String STORE = "store";
	public static final String LOAD = "load";
	public static final String CLEAR = "clear";
	public static final String NEXT_BOND = "nextBond";
	public static final String INSTALL = "install";
	public static final String ECLIPSE = "eclipse";
	public static final String SET_PATH = "setPath";
	public static final String GET_PATH = "path";
	public static final String MERGINGS = "mergings";
	//fields
	protected Bridge _bridge = null;
	protected int _count = 0;
	protected URI _eclipsePath = URI.createFileURI("");
	private CoreEvaluator _eval = null;
	
	public CoreRunner(){
		_eval = new CoreEvaluator(this);
		run();
	}
	public void run(){
		BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
		greet();
		
		System.out.println("Loading conexp-clj...");
		System.out.println("  Version: " +conexp_version());
		try {
			File pathFile = new File(".path");
			if(pathFile.exists()){
				BufferedReader rd = new BufferedReader(new FileReader(pathFile));
				String p = rd.readLine();
				File f = new File(p);
				if (f.exists()) {
					_eclipsePath = URI.createFileURI(p);
				} else {
					System.out.println("Invalid path to eclipse. Set it manually!");
				}
				rd.close();
			}
			//set creator
			_bridge = new Bridge();
			while(true) {
				mainLoop(reader);
			}
		} catch (Exception e) {
			System.out.println("  " + e.getClass().getCanonicalName());
			e.printStackTrace();
			if(e.getMessage()!=null){
				try {
					PrintWriter pw = new PrintWriter(new FileWriter("error.log"));
					pw.write(GregorianCalendar.getInstance().getTime().toString() + "\n");
					pw.write(e.getLocalizedMessage() + "\n");
					pw.write("\n");
					pw.flush();
					pw.close();
				} catch (IOException e1) {
					System.out.println("  Error writing log file");
				}
			}
			_eval.exit(true);
		}	
	}
	private void mainLoop(BufferedReader reader) throws Exception{
		while(true){
			System.out.print(PRE);
			evalCommand(reader);
		}
	}
	private void greet(){
		System.out.println("\n#####################################################");
		System.out.println("# Welcome to the Core - the Contextual Role Editor. #");
		System.out.println("#                 -- Ver. 1.0.1 --                  #");
		System.out.println("# ------------------------------------------------- #");
		System.out.println("# Contact: henri.muehle@univie.ac.at                #");
		System.out.println("# (c) Henri Muehle, 2010-2011.                      #");
		System.out.println("#####################################################\n");
	}
	private void evalCommand(BufferedReader reader) throws Exception{
		String in = reader.readLine();
		String[] commands = null;
		//separate command from arguments
		commands = evalInput(in.split(" "));
		//evaluate commands
		if (commands.length==0) return;
		if (commands[0].equals(EXIT)) _eval.exit(false);
		else if (commands[0].equals(HELP)) _eval.help();
		else if (commands[0].equals(CLEAR)) _eval.clear();
		else if (commands[0].equals(SET_PATH)) _eval.setPath(commands);
		else if (commands[0].equals(GET_PATH)) _eval.getPath();
		else if (commands[0].equals(ECLIPSE)) _eval.runEclipse();
		else if (commands[0].equals(INSTALL)) _eval.installEditor(commands);
		else if (commands[0].equals(PRINT)) _eval.print(commands);
		else if (commands[0].equals(STORE)) _eval.store(commands);
		else if (commands[0].equals(LOAD)) _eval.load(commands);
		else if (commands[0].equals(NEXT_BOND)) _eval.nextBond(reader);
		else if (commands[0].equals(MERGINGS)) _eval.mergings(commands);
		else _eval.notValid();
	}
	private String[] evalInput(String[] commands) {
		int c = 0;
		for (String s : commands) {
			if (!s.equals("")) c++;
		}
		String[] res = new String[c];
		c = 0;
		for (String s : commands) {
			if(!s.equals("")){
				res[c] = s;
				c++;
			}
		}
		return res;
	}
	public static void main(String[] args) {
		CoreRunner runner = new CoreRunner();
		runner.run();
	}
}
