package net.core.editor.roles.diagram.custom.util;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import net.core.editor.roles.Attribute;
import net.core.editor.roles.BaseInheritanceEdge;
import net.core.editor.roles.BaseType;
import net.core.editor.roles.Method;
import net.core.editor.roles.RoleInheritanceEdge;
import net.core.editor.roles.RoleModel;
import net.core.editor.roles.RoleModelPackage;
import net.core.editor.roles.RolePlayEdge;
import net.core.editor.roles.RoleType;
import net.core.editor.roles.Type;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import static conexp.contrib.java.Main.*;

public class ContextGenerator {
	private RoleModel roleModel;
	private URI _path;
	private Bridge _bridge;
	//katalogisierte Inhalte des Rollenmodells
	private HashMap<Type, Integer> types = new HashMap<Type, Integer>();
	private HashMap<Integer, List<Attribute>> attributes = new HashMap<Integer, List<Attribute>>();
	private HashMap<Integer, List<Method>> methods = new HashMap<Integer, List<Method>>();
	private HashMap<Type, Type> inheritsFrom = new HashMap<Type, Type>();
	private HashMap<RoleType, BaseType> playedBy = new HashMap<RoleType, BaseType>();

	public ContextGenerator(){
		super();
		_path = URI.createURI("");
	}
	public ContextGenerator(Bridge bridge) {
		this();
		_bridge = bridge;
	}
	public void clear() {
		roleModel = null;
		types.clear();
		attributes.clear();
		methods.clear();
		inheritsFrom.clear();
		playedBy.clear();
	}
	public void initRoleModel() {
		//try opening
		try {
			new FileReader(_path.toFileString());
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		//initialize model
		RoleModelPackage.eINSTANCE.eClass();
		//register XMI resource for .rd extension
		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map<String, Object> m = reg.getExtensionToFactoryMap();
		m.put("rd", new XMIResourceFactoryImpl());
		//obtain a new resource set
		ResourceSet resourceSet = new ResourceSetImpl();
		//get the resource
		Resource resource = resourceSet.getResource(_path, true);
		//get the role model
		roleModel = (RoleModel) resource.getContents().get(0);
	}
	public void execute() {
		indexData();

		generateBasicContexts();
		generateRolePlayContext();
		generateCompositionContext(_bridge.getContext(Bridge.BASE_CONTEXT), _bridge.getContext(Bridge.ROLE_CONTEXT));
	}
	public void setPath(URI path) {
		_path = path;
	}
	public void indexData() {
		if(roleModel == null){
			initRoleModel();			
		}
		
		int c = 0;
		//katalogisiere die Typen und ihre eigenen Felder
		TreeIterator<EObject> treeIt = roleModel.eAllContents();
		while(treeIt.hasNext()){
			Object o = treeIt.next();
			if(o instanceof Type){
				types.put((Type) o, c);
				List<Method> ml = new LinkedList<Method>();
				for(Method m : ((Type) o).getMethods()){
					ml.add(m);
				}
				methods.put(c, ml);
				List<Attribute> al = new LinkedList<Attribute>();
				for(Attribute a : ((Type) o).getAttributes()){
					al.add(a);
				}
				attributes.put(c, al);
				c++;
			}
		}		
		//katalogisiere die Vererbungsrelationen		
		treeIt = roleModel.eAllContents();
		while(treeIt.hasNext()){
			Object o = treeIt.next();
			if(o instanceof BaseInheritanceEdge || o instanceof RoleInheritanceEdge){
				Type source = (o instanceof BaseInheritanceEdge) ? ((BaseInheritanceEdge) o).getSource() : ((RoleInheritanceEdge) o).getSource();
				Type target = (o instanceof BaseInheritanceEdge) ? ((BaseInheritanceEdge) o).getTarget() : ((RoleInheritanceEdge) o).getTarget();
				int s = types.get(source);
				int t = types.get(target);
				
				List<Attribute> sourceAttributes = attributes.get(s);
				for(Attribute a : attributes.get(t)){
					if(!sourceAttributes.contains(a)){
						sourceAttributes.add(a);
					}
				}
				List<Method> sourceMethods = methods.get(s);
				for(Method m : methods.get(t)){
					if(!sourceMethods.contains(m)){
						sourceMethods.add(m);
					}
				}
				inheritsFrom.put(source, target);
			}
			if(o instanceof RolePlayEdge){
				playedBy.put(((RolePlayEdge) o).getSource(), ((RolePlayEdge) o).getTarget());
			}
		}
	}
	@SuppressWarnings("unchecked")
	public void generateBasicContexts(){
		for(int k=0; k<2; k++){
			LinkedList G = new LinkedList();
			LinkedList Ma = new LinkedList();
			LinkedList Mm = new LinkedList();
			for(Type t : types.keySet()){
				if((t instanceof BaseType && k==0) || (t instanceof RoleType && k==1)){
					G.add(t.getName());
					for(Attribute a : attributes.get(types.get(t))){
						String n = a.getName() + ":" + a.getType();
						if(!Ma.contains(n)){
							Ma.add(n);
						}
					}
					for(Method m : methods.get(types.get(t))){
						String n = m.getName() + "():" + m.getReturnType();
						if(!Mm.contains(n)){
							Mm.add(n);
						}
					}
				}
			}
			LinkedList M = new LinkedList();
			M.addAll(Ma);
			M.addAll(Mm);
			int[] I = new int[G.size()*M.size()];
			for(Type t : types.keySet()){
				int i = G.indexOf(t.getName());
				if(i!=-1){
					for(Attribute a : attributes.get(types.get(t))){
						int j = Ma.indexOf(a.getName() + ":" + a.getType());
						if(j!=-1){
							I[i*M.size()+j] = 1;
						}
					}
					for(Method m : methods.get(types.get(t))){
						int j = Mm.indexOf(m.getName() + "():" + m.getReturnType());
						if(j!=-1){
							I[i*M.size()+Ma.size()+j] = 1;
						}
					}
				}
			}
			if (k==0) {
				_bridge.setContext(Bridge.BASE_CONTEXT, make_context_from_matrix(G.toArray(), M.toArray(), I));
			} else {
				_bridge.setContext(Bridge.ROLE_CONTEXT, make_context_from_matrix(G.toArray(), M.toArray(), I));
			}
		}	
	}
	@SuppressWarnings("unchecked")
	public void generateRolePlayContext(){
		LinkedList B = new LinkedList();
		LinkedList R = new LinkedList();
		for (Type t : types.keySet()) {
			if (t instanceof RoleType) R.add(t.getName());
			if (t instanceof BaseType) B.add(t.getName());
		}
		int[] P = new int[B.size()*R.size()];
		for (RoleType r : playedBy.keySet()) {
			BaseType b = playedBy.get(r);
			P[B.indexOf(b.getName())*R.size()+R.indexOf(r.getName())] = 1;
			
			Collection gbi = new LinkedList();
			for (Object o : (Collection) objects(_bridge.getContext(Bridge.BASE_CONTEXT))) {
				if (Bridge.leqObj(_bridge.getContext(Bridge.BASE_CONTEXT), o, b.getName())) gbi.add(o);
			}
			Collection grf = new LinkedList();
			for (Object o : (Collection) objects(_bridge.getContext(Bridge.ROLE_CONTEXT))) {
				if (Bridge.leqObj(_bridge.getContext(Bridge.ROLE_CONTEXT), r.getName(), o)) grf.add(o);
			}
			for (Object hb : gbi) {
				for (Object hr : grf) {
					P[B.indexOf(hb)*R.size()+R.indexOf(hr)] = 1;
				}
			}
		}
		_bridge.setContext(Bridge.ROLE_PLAY, make_context_from_matrix(B,R,P));
	}
	public void generateContraordinalScales(){
		if(_bridge.getContext(Bridge.BASE_CONTEXT)==null || _bridge.getContext(Bridge.ROLE_CONTEXT)==null){
			generateBasicContexts();
		}
		_bridge.setContext(Bridge.BASE_SCALE, Bridge.contraordinalScale(_bridge.getContext(Bridge.BASE_CONTEXT)));
		_bridge.setContext(Bridge.ROLE_SCALE, Bridge.contraordinalScale(_bridge.getContext(Bridge.ROLE_CONTEXT)));
	}
	@SuppressWarnings("unchecked")
	public void generateCompositionContext(Object B, Object R){
		Collection base_objects = (Collection) objects(B);
		Collection base_attributes = (Collection) attributes(B);
		Collection role_objects = (Collection) objects(R);
		Collection role_attributes = (Collection) attributes(R);
		
		int[][] IB = new int[base_objects.size()][base_attributes.size()];
		for (int i=0; i<base_objects.size(); i++) {
			Object b = Bridge.getFromCollection(base_objects, i);
			for (int j=0; j<base_attributes.size(); j++) {
				Object ba = Bridge.getFromCollection(base_attributes, j);
				if(incident_p(B, b, ba)!=null) IB[i][j] = 1;
			}
		}
		int[][] IR = new int[role_objects.size()][role_attributes.size()];
		for (int i=0; i<role_objects.size(); i++) {
			Object r = Bridge.getFromCollection(role_objects, i);
			for (int j=0; j<role_attributes.size(); j++) {
				Object ra = Bridge.getFromCollection(role_attributes, j);
				if(incident_p(R, r, ra)!=null) IR[i][j] = 1;
			}
		}
		int[][] IZ = new int[base_objects.size()][role_attributes.size()];
//		Object Z = make_context_from_matrix(base_objects, role_attributes, IZ);
		
		int[][] ID = new int[role_objects.size()][base_attributes.size()];
		for (int i=0; i<role_objects.size(); i++) {
			Object r = Bridge.getFromCollection(role_objects, i);
			for (int j=0; j<base_attributes.size(); j++) {
				Object ba = Bridge.getFromCollection(base_attributes, j);
				for (Object b : base_objects) {
					if (incident_p(_bridge.getContext(Bridge.ROLE_PLAY), b, r)!=null && incident_p(B, b, ba)!=null) {
						ID[i][j] = 1;
						break;
					}
				}
			}
		}
		Collection G = new LinkedList();
		G.addAll(base_objects);
		G.addAll(role_objects);
		Collection M = new LinkedList();
		M.addAll(base_attributes);
		M.addAll(role_attributes);
		int[] I = new int[G.size()*M.size()];
		
		for (int i=0; i<G.size(); i++) {
			for (int j=0; j<M.size(); j++) {
				if (i<base_objects.size() && j<base_attributes.size()) {
					I[i*(base_objects.size()+role_objects.size())+j] = IB[i][j];
				}
				if (i<base_objects.size() && j>=base_attributes.size()) {
					I[i*(base_objects.size()+role_objects.size())+j] = IZ[i][j-base_attributes.size()];
				}
				if (i>=base_objects.size() && j<base_attributes.size()) {
					I[i*(base_objects.size()+role_objects.size())+j] = ID[i-base_objects.size()][j];
				}
				if (i>=base_objects.size() && j>=base_attributes.size()) {
					I[i*(base_objects.size()+role_objects.size())+j] = IR[i-base_objects.size()][j-base_attributes.size()];
				}
			}
		}
		
		Object C = make_context_from_matrix(G.toArray(), M.toArray(), I);
		_bridge.setContext(Bridge.COMPOSITION_CONTEXT, C);
	}
}

